library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.am2901_comp.all;
use work.am2901_mnemonics.all;

entity alu_source_mux is

  generic (
    WIDTH : integer := 4);

  port (
    Direct_Data_in       : in  std_logic_vector((WIDTH - 1) downto 0);
    R_in                 : in  std_logic_vector((WIDTH - 1) downto 0);
    S_in                 : in  std_logic_vector((WIDTH - 1) downto 0);
    Q_Reg_in             : in  std_logic_vector((WIDTH - 1) downto 0);
    ALU_Source_Select_in : in  std_logic_vector(2 downto 0);
    MUX_A_out            : out std_logic_vector((WIDTH - 1) downto 0);
    MUX_B_out            : out std_logic_vector((WIDTH - 1) downto 0));

end alu_source_mux;

architecture arch_alu_source_mux of alu_source_mux is

  signal Direct_Data       : std_logic_vector((WIDTH - 1) downto 0);
  signal R                 : std_logic_vector((WIDTH - 1) downto 0);
  signal S                 : std_logic_vector((WIDTH - 1) downto 0);
  signal Q_Reg             : std_logic_vector((WIDTH - 1) downto 0);
  signal ALU_Source_Select : std_logic_vector(2 downto 0);
  signal MUX_A             : std_logic_vector((WIDTH - 1) downto 0);
  signal MUX_B             : std_logic_vector((WIDTH - 1) downto 0);

begin  -- arch_alu_source_mux

  Direct_Data       <= Direct_Data_in;
  R                 <= R_in;
  S                 <= S_in;
  Q_Reg             <= Q_Reg_in;
  ALU_Source_Select <= ALU_Source_Select_in;

  MUX_A_out <= MUX_A;
  MUX_B_out <= MUX_B;

  U1 : process (Direct_Data, R, S, Q_Reg, ALU_Source_Select)
  begin  -- process U1

    case ALU_Source_Select is
      when AQ =>

        MUX_A <= R;
        MUX_B <= Q_Reg;

      when AB =>

        MUX_A <= R;
        MUX_B <= S;

      when ZQ =>

        MUX_A <= (others => '0');
        MUX_B <= Q_Reg;

      when ZB =>

        MUX_A <= (others => '0');
        MUX_B <= S;

      when ZA =>

        MUX_A <= (others => '0');
        MUX_B <= R;

      when DA =>

        MUX_A <= Direct_Data;
        MUX_B <= R;

      when DQ =>

        MUX_A <= Direct_Data;
        MUX_B <= Q_reg;

      when DZ =>

        MUX_A <= Direct_Data;
        MUX_B <= (others => '0');

      when others => null;

    end case;  -- case ALU_Source_Select

  end process U1;

end arch_alu_source_mux;


