library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

package am2901_comp is

  component ram_shift
    generic (
      WIDTH              :     integer);
    port (
      RAM_Shift_in       : in  std_logic_vector((WIDTH - 1) downto 0);
      RAM_Shift_Cntrl_in : in  std_logic_vector(2 downto 0);
      RAM_LSB_in         : in  std_logic;
      RAM_MSB_in         : in  std_logic;
      RAM_LSB_out        : out std_logic;
      RAM_MSB_out        : out std_logic;
      RAM_LSB_Dir_out    : out std_logic;
      RAM_MSB_Dir_out    : out std_logic;
      RAM_Shift_out      : out std_logic_vector((WIDTH - 1) downto 0));
  end component;

  component reg_ram
    generic (
      WIDTH        :     integer);
    port (
      Clock_in     : in  std_logic;
      Data_in      : in  std_logic_vector((WIDTH - 1) downto 0);
      A_Address_in : in  std_logic_vector(3 downto 0);
      B_Address_in : in  std_logic_vector(3 downto 0);
      RAM_WE_in    : in  std_logic;
      A_Data_out   : out std_logic_vector((WIDTH - 1) downto 0);
      B_Data_out   : out std_logic_vector((WIDTH - 1) downto 0));
  end component;

  component alu_source_mux
    generic (
      WIDTH                :     integer);
    port (
      Direct_Data_in       : in  std_logic_vector((WIDTH - 1) downto 0);
      R_in                 : in  std_logic_vector((WIDTH - 1) downto 0);
      S_in                 : in  std_logic_vector((WIDTH - 1) downto 0);
      Q_Reg_in             : in  std_logic_vector((WIDTH - 1) downto 0);
      ALU_Source_Select_in : in  std_logic_vector(2 downto 0);
      MUX_A_out            : out std_logic_vector((WIDTH - 1) downto 0);
      MUX_B_out            : out std_logic_vector((WIDTH - 1) downto 0));
  end component;

  component q_shift
    generic (
      WIDTH            :     integer);
    port (
      Q_Shift_in       : in  std_logic_vector((WIDTH - 1) downto 0);
      F_in             : in  std_logic_vector((WIDTH - 1) downto 0);
      Q_Shift_Cntrl_in : in  std_logic_vector(2 downto 0);
      Q_LSB_in         : in  std_logic;
      Q_MSB_in         : in  std_logic;
      Q_LSB_out        : out std_logic;
      Q_MSB_out        : out std_logic;
      Q_LSB_Dir_out    : out std_logic;
      Q_MSB_Dir_out    : out std_logic;
      Q_Shift_out      : out std_logic_vector((WIDTH - 1) downto 0));
  end component;

  component q_register
    generic (
      WIDTH      :     integer);
    port (
      Clock_in   : in  std_logic;
      Q_Shift_in : in  std_logic_vector((WIDTH - 1) downto 0);
      Q_out      : out std_logic_vector((WIDTH - 1) downto 0));
  end component;

  component alu
    generic (
      WIDTH        :     integer);
    port (
      R_in         : in  std_logic_vector((WIDTH - 1) downto 0);
      S_in         : in  std_logic_vector((WIDTH - 1) downto 0);
      CI_in        : in  std_logic;
      ALU_Cntrl_in : in  std_logic_vector(2 downto 0);
      F_out        : out std_logic_vector((WIDTH - 1) downto 0);
      G_out        : out std_logic;
      P_out        : out std_logic;
      CO_out       : out std_logic;
      F_MSB_out    : out std_logic;
      OVR_out      : out std_logic;
      Feq0_out     : out std_logic);
  end component;

  component output_mux
    generic (
      WIDTH            :     integer);
    port (
      A_Data_in        : in  std_logic_vector((WIDTH - 1) downto 0);
      F_in             : in  std_logic_vector((WIDTH - 1) downto 0);
      Output_Select_in : in  std_logic_vector(2 downto 0);
      OE_in            : in  std_logic;
      Y_out            : out std_logic_vector((WIDTH - 1) downto 0));
  end component;

  component instruction_decode
    port (
      I_in                  : in  std_logic_vector(8 downto 0);
      RAM_WE_out            : out std_logic;
      RAM_Shift_Cntrl_out   : out std_logic_vector(2 downto 0);
      Q_Shift_Cntrl_out     : out std_logic_vector(2 downto 0);
      ALU_Cntrl_out         : out std_logic_vector(2 downto 0);
      ALU_Source_Select_out : out std_logic_vector(2 downto 0);
      Output_Select_out     : out std_logic_vector(2 downto 0));
  end component;

end am2901_comp;
