library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.am2901_comp.all;
use work.am2901_mnemonics.all;

entity ram_shift is

  generic (
    WIDTH : integer := 4);

  port (
    RAM_Shift_in       : in  std_logic_vector((WIDTH - 1) downto 0);
    RAM_Shift_Cntrl_in : in  std_logic_vector(2 downto 0);
    RAM_LSB_in         : in  std_logic;
    RAM_MSB_in         : in  std_logic;
    RAM_LSB_out        : out std_logic;
    RAM_MSB_out        : out std_logic;
    RAM_LSB_Dir_out    : out std_logic;
    RAM_MSB_Dir_out    : out std_logic;
    RAM_Shift_out      : out std_logic_vector((WIDTH - 1) downto 0));

end ram_shift;

architecture arch_ram_shift of ram_shift is


  signal RAM_Shift_Cntrl : std_logic_vector(2 downto 0);
  signal RAM_LSB_Dir     : std_logic;
  signal RAM_MSB_Dir     : std_logic;

begin  -- arch_ram_shift

  RAM_Shift_Cntrl <= RAM_Shift_Cntrl_in;

  RAM_LSB_Dir_out <= RAM_LSB_Dir;
  RAM_MSB_Dir_out <= RAM_MSB_Dir;

  -- RAM_xxx_Dir = '1' direction is input
  -- RAM_xxx_Dir = '0' direction is output
  -- xxx = LSB or MSB

  U1 : process (RAM_Shift_in, RAM_Shift_Cntrl, RAM_LSB_in, RAM_MSB_in)
  begin  -- process U1

    case RAM_Shift_Cntrl is

      when QREG =>
        RAM_Shift_out <= RAM_Shift_in;

        RAM_LSB_out <= '0';
        RAM_LSB_Dir <= '1';

        RAM_MSB_out <= '0';
        RAM_MSB_Dir <= '1';

      when NOP =>
        RAM_Shift_out <= RAM_Shift_in;

        RAM_LSB_out <= '0';
        RAM_LSB_Dir <= '1';

        RAM_MSB_out <= '0';
        RAM_MSB_Dir <= '1';


      when RAMA =>
        RAM_Shift_out <= RAM_Shift_in;

        RAM_LSB_out <= '0';
        RAM_LSB_Dir <= '1';

        RAM_MSB_out <= '0';
        RAM_MSB_Dir <= '1';

      when RAMF =>
        RAM_Shift_out <= RAM_Shift_in;

        RAM_LSB_out <= '0';
        RAM_LSB_Dir <= '1';

        RAM_MSB_out <= '0';
        RAM_MSB_Dir <= '1';

      when RAMQD =>
        RAM_Shift_out(WIDTH - 1) <= RAM_MSB_in;
        for ramqd_index in (WIDTH - 2) downto 0 loop

          RAM_Shift_out(ramqd_index) <= RAM_Shift_in(ramqd_index + 1);

        end loop;  -- ramqd_index

        RAM_LSB_out <= RAM_Shift_in(0);
        RAM_LSB_Dir <= '0';

        RAM_MSB_out <= '0';
        RAM_MSB_Dir <= '1';

      when RAMD =>
        RAM_Shift_out(WIDTH - 1) <= RAM_MSB_in;
        for ramd_index in (WIDTH - 2) downto 0 loop

          RAM_Shift_out(ramd_index) <= RAM_Shift_in(ramd_index + 1);

        end loop;  -- ramd_index

        RAM_LSB_out <= RAM_Shift_in(0);
        RAM_LSB_Dir <= '0';

        RAM_MSB_out <= '0';
        RAM_MSB_Dir <= '1';

      when RAMQU =>
        for ramqu_index in (WIDTH - 1) downto 1 loop

          RAM_Shift_out(ramqu_index) <= RAM_Shift_in(ramqu_index - 1);

        end loop;  -- ramqu_index
        RAM_Shift_out(0) <= RAM_LSB_in;

        RAM_LSB_out <= '0';
        RAM_LSB_Dir <= '1';

        RAM_MSB_out <= RAM_Shift_in((WIDTH - 1));
        RAM_MSB_Dir <= '0';

      when RAMU =>
        for ramu_index in (WIDTH - 1) downto 1 loop

          RAM_Shift_out(ramu_index) <= RAM_Shift_in(ramu_index - 1);

        end loop;  -- ramu_index
        RAM_Shift_out(0) <= RAM_LSB_in;

        RAM_LSB_out <= '0';
        RAM_LSB_Dir <= '1';

        RAM_MSB_out <= RAM_Shift_in((WIDTH - 1));
        RAM_MSB_Dir <= '0';

      when others => null;

    end case;

  end process U1;

end arch_ram_shift;
