library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

package am2910_comp is

  component reg_count
    port (
      Clock_in       : in  std_logic;
      Direct_Load_in : in  std_logic;
      Decr_En_in     : in  std_logic;
      Load_En_in     : in  std_logic;
      D_in           : in  std_logic_vector(11 downto 0);
      Q_out          : out std_logic_vector(11 downto 0));
  end component;

  component zero_detector
    port (
      D_in     : in  std_logic_vector(11 downto 0);
      Zero_out : out std_logic);
  end component;

  component stack
    generic (
      DEPTH     :     integer := 9);
    port (
      Clock_in  : in  std_logic;
      Stack_in  : in  std_logic_vector(11 downto 0);
      Clr_in    : in  std_logic;
      Push_in   : in  std_logic;
      Pop_in    : in  std_logic;
      Empty_out : out std_logic;
      Full_out  : out std_logic;
      Stack_out : out std_logic_vector(11 downto 0));
  end component;

  component stack_pointer
    generic (
      DEPTH       :     integer := 9);
    port (
      Clock_in    : in  std_logic;
      Clr_in      : in  std_logic;
      Push_in     : in  std_logic;
      Pop_in      : in  std_logic;
      Empty_out   : out std_logic;
      Full_out    : out std_logic;
      WR_Addr_out : out std_logic_vector(3 downto 0);
      RD_Addr_out : out std_logic_vector(3 downto 0));
  end component;

  component stack_ram
    port (
      WR_CLK_in   : in  std_logic;
      WR_Addr_in  : in  std_logic_vector(3 downto 0);
      RD_Addr_in  : in  std_logic_vector(3 downto 0);
      Stack_WE_in : in  std_logic;
      WR_Data_in  : in  std_logic_vector(11 downto 0);
      RD_Data_out : out std_logic_vector(11 downto 0));
  end component;

  component multiplexer
    port (
      D0_in      : in  std_logic_vector(11 downto 0);
      D1_in      : in  std_logic_vector(11 downto 0);
      D2_in      : in  std_logic_vector(11 downto 0);
      D3_in      : in  std_logic_vector(11 downto 0);
      Mux_Sel_in : in  std_logic_vector(1 downto 0);
      Mux_En_in  : in  std_logic;
      Mux_out    : out std_logic_vector(11 downto 0));
  end component;

  component program_counter
    port (
      Clock_in : in  std_logic;
      D_in     : in  std_logic_vector(11 downto 0);
      Q_out    : out std_logic_vector(11 downto 0));
  end component;

  component incrementer
    port (
      A_in     : in  std_logic_vector(11 downto 0);
      Carry_in : in  std_logic;
      Sum_out  : out std_logic_vector(11 downto 0));
  end component;

  component instruction_pla
    port (
      Cond_Code_in    : in  std_logic;
      Cond_Code_En_in : in  std_logic;
      Instr_in        : in  std_logic_vector(3 downto 0);
      RegZero_in      : in  std_logic;
      Mux_Sel_out     : out std_logic_vector(1 downto 0);
      Mux_En_out      : out std_logic;
      Reg_Decr_out    : out std_logic;
      Reg_Load_out    : out std_logic;
      Stk_Clr_out     : out std_logic;
      Stk_Push_out    : out std_logic;
      Stk_Pop_out     : out std_logic;
      PL_En_out       : out std_logic;
      Map_En_out      : out std_logic;
      Vect_En_out     : out std_logic);
  end component;


end package;
