library ieee;
use ieee.std_logic_1164.all;

package am2910_const is

-- Micro-instruction Mnemonics

  constant uinst_JZ   : std_logic_vector(3 downto 0) := B"0000";
  constant uinst_CJS  : std_logic_vector(3 downto 0) := B"0001";
  constant uinst_JMAP : std_logic_vector(3 downto 0) := B"0010";
  constant uinst_CJP  : std_logic_vector(3 downto 0) := B"0011";
  constant uinst_PUSH : std_logic_vector(3 downto 0) := B"0100";
  constant uinst_JSRP : std_logic_vector(3 downto 0) := B"0101";
  constant uinst_CJV  : std_logic_vector(3 downto 0) := B"0110";
  constant uinst_JRP  : std_logic_vector(3 downto 0) := B"0111";
  constant uinst_RFCT : std_logic_vector(3 downto 0) := B"1000";
  constant uinst_RPCT : std_logic_vector(3 downto 0) := B"1001";
  constant uinst_CRTN : std_logic_vector(3 downto 0) := B"1010";
  constant uinst_CJPP : std_logic_vector(3 downto 0) := B"1011";
  constant uinst_LDCT : std_logic_vector(3 downto 0) := B"1100";
  constant uinst_LOOP : std_logic_vector(3 downto 0) := B"1101";
  constant uinst_CONT : std_logic_vector(3 downto 0) := B"1110";
  constant uinst_TWB  : std_logic_vector(3 downto 0) := B"1111";

-- Multiplexer Sources

  constant mux_uPC    : std_logic_vector(2 downto 0) := B"100";
  constant mux_STACK  : std_logic_vector(2 downto 0) := B"101";
  constant mux_REG    : std_logic_vector(2 downto 0) := B"110";
  constant mux_DIRECT : std_logic_vector(2 downto 0) := B"111";
  constant mux_ZERO   : std_logic_vector(2 downto 0) := B"000";

-- Stack Operations

  constant stack_PUSH  : std_logic_vector(2 downto 0) := B"010";
  constant stack_POP   : std_logic_vector(2 downto 0) := B"001";
  constant stack_HOLD  : std_logic_vector(2 downto 0) := B"000";
  constant stack_CLEAR : std_logic_vector(2 downto 0) := B"100";

-- Register-Counter Operations

  constant register_HOLD : std_logic_vector(1 downto 0) := B"00";
  constant register_DECR : std_logic_vector(1 downto 0) := B"10";
  constant register_LOAD : std_logic_vector(1 downto 0) := B"01";

  constant REGeqZERO : std_logic := '1';

-- Program Counter Operations

  constant pc_LOAD  : std_logic := '0';
  constant pc_CLEAR : std_logic := '1';

-- Pipeline Enable

  constant pl_ENABLE  : std_logic := '1';
  constant pl_DISABLE : std_logic := '0';

-- MAP Enable

  constant map_ENABLE  : std_logic := '1';
  constant map_DISABLE : std_logic := '0';

-- Vector Enable

  constant vect_ENABLE  : std_logic := '1';
  constant vect_DISABLE : std_logic := '0';

end package;
