library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.all;

entity stack_ram is
  port (
    WR_CLK_in   : in  std_logic;
    WR_Addr_in  : in  std_logic_vector(3 downto 0);
    RD_Addr_in  : in  std_logic_vector(3 downto 0);
    Stack_WE_in : in  std_logic;
    WR_Data_in  : in  std_logic_vector(11 downto 0);
    RD_Data_out : out std_logic_vector(11 downto 0));
end stack_ram;

architecture arch_stack_ram of stack_ram is

  component RAM16X1D
    generic (
      INIT  :     bit_vector(15 downto 0));
    port (
      DPO   : out std_ulogic;
      SPO   : out std_ulogic;
      A0    : in  std_ulogic;
      A1    : in  std_ulogic;
      A2    : in  std_ulogic;
      A3    : in  std_ulogic;
      D     : in  std_ulogic;
      DPRA0 : in  std_ulogic;
      DPRA1 : in  std_ulogic;
      DPRA2 : in  std_ulogic;
      DPRA3 : in  std_ulogic;
      WCLK  : in  std_ulogic;
      WE    : in  std_ulogic);
  end component;

  signal WR_CLK   : std_logic;
  signal WR_Addr  : std_logic_vector(3 downto 0);
  signal RD_Addr  : std_logic_vector(3 downto 0);
  signal Stack_WE : std_logic;
  signal WR_Data  : std_logic_vector(11 downto 0);
  signal RD_Data  : std_logic_vector(11 downto 0);
  
  signal RAM_WR_out : std_logic_vector(11 downto 0);

begin  -- arch_stack_ram

WR_CLK   <= WR_CLK_in;
WR_Addr  <= WR_Addr_in;
RD_Addr  <= RD_Addr_in;
Stack_WE <= Stack_WE_in;
WR_Data  <= WR_Data_in;

RD_Data_out <= RD_Data;

  U0 : RAM16X1D
    generic map (
      INIT => X"0000")
    port map (
      DPO   => RD_Data(0),
      SPO   => RAM_WR_out(0),
      A0    => WR_Addr(0),
      A1    => WR_Addr(1),
      A2    => WR_Addr(2),
      A3    => WR_Addr(3),
      D     => WR_Data(0),
      DPRA0 => RD_Addr(0),
      DPRA1 => RD_Addr(1),
      DPRA2 => RD_Addr(2),
      DPRA3 => RD_Addr(3),
      WCLK  => WR_CLK,
      WE    => Stack_WE);

  U1 : RAM16X1D
    generic map (
      INIT => X"0000")
    port map (
      DPO   => RD_Data(1),
      SPO   => RAM_WR_out(1),
      A0    => WR_Addr(0),
      A1    => WR_Addr(1),
      A2    => WR_Addr(2),
      A3    => WR_Addr(3),
      D     => WR_Data(1),
      DPRA0 => RD_Addr(0),
      DPRA1 => RD_Addr(1),
      DPRA2 => RD_Addr(2),
      DPRA3 => RD_Addr(3),
      WCLK  => WR_CLK,
      WE    => Stack_WE);

  U2 : RAM16X1D
    generic map (
      INIT => X"0000")
    port map (
      DPO   => RD_Data(2),
      SPO   => RAM_WR_out(2),
      A0    => WR_Addr(0),
      A1    => WR_Addr(1),
      A2    => WR_Addr(2),
      A3    => WR_Addr(3),
      D     => WR_Data(2),
      DPRA0 => RD_Addr(0),
      DPRA1 => RD_Addr(1),
      DPRA2 => RD_Addr(2),
      DPRA3 => RD_Addr(3),
      WCLK  => WR_CLK,
      WE    => Stack_WE);

  U3 : RAM16X1D
    generic map (
      INIT => X"0000")
    port map (
      DPO   => RD_Data(3),
      SPO   => RAM_WR_out(3),
      A0    => WR_Addr(0),
      A1    => WR_Addr(1),
      A2    => WR_Addr(2),
      A3    => WR_Addr(3),
      D     => WR_Data(3),
      DPRA0 => RD_Addr(0),
      DPRA1 => RD_Addr(1),
      DPRA2 => RD_Addr(2),
      DPRA3 => RD_Addr(3),
      WCLK  => WR_CLK,
      WE    => Stack_WE);

  U4 : RAM16X1D
    generic map (
      INIT => X"0000")
    port map (
      DPO   => RD_Data(4),
      SPO   => RAM_WR_out(4),
      A0    => WR_Addr(0),
      A1    => WR_Addr(1),
      A2    => WR_Addr(2),
      A3    => WR_Addr(3),
      D     => WR_Data(4),
      DPRA0 => RD_Addr(0),
      DPRA1 => RD_Addr(1),
      DPRA2 => RD_Addr(2),
      DPRA3 => RD_Addr(3),
      WCLK  => WR_CLK,
      WE    => Stack_WE);

  U5 : RAM16X1D
    generic map (
      INIT => X"0000")
    port map (
      DPO   => RD_Data(5),
      SPO   => RAM_WR_out(5),
      A0    => WR_Addr(0),
      A1    => WR_Addr(1),
      A2    => WR_Addr(2),
      A3    => WR_Addr(3),
      D     => WR_Data(5),
      DPRA0 => RD_Addr(0),
      DPRA1 => RD_Addr(1),
      DPRA2 => RD_Addr(2),
      DPRA3 => RD_Addr(3),
      WCLK  => WR_CLK,
      WE    => Stack_WE);

  U6 : RAM16X1D
    generic map (
      INIT => X"0000")
    port map (
      DPO   => RD_Data(6),
      SPO   => RAM_WR_out(6),
      A0    => WR_Addr(0),
      A1    => WR_Addr(1),
      A2    => WR_Addr(2),
      A3    => WR_Addr(3),
      D     => WR_Data(6),
      DPRA0 => RD_Addr(0),
      DPRA1 => RD_Addr(1),
      DPRA2 => RD_Addr(2),
      DPRA3 => RD_Addr(3),
      WCLK  => WR_CLK,
      WE    => Stack_WE);

  U7 : RAM16X1D
    generic map (
      INIT => X"0000")
    port map (
      DPO   => RD_Data(7),
      SPO   => RAM_WR_out(7),
      A0    => WR_Addr(0),
      A1    => WR_Addr(1),
      A2    => WR_Addr(2),
      A3    => WR_Addr(3),
      D     => WR_Data(7),
      DPRA0 => RD_Addr(0),
      DPRA1 => RD_Addr(1),
      DPRA2 => RD_Addr(2),
      DPRA3 => RD_Addr(3),
      WCLK  => WR_CLK,
      WE    => Stack_WE);

  U8 : RAM16X1D
    generic map (
      INIT => X"0000")
    port map (
      DPO   => RD_Data(8),
      SPO   => RAM_WR_out(8),
      A0    => WR_Addr(0),
      A1    => WR_Addr(1),
      A2    => WR_Addr(2),
      A3    => WR_Addr(3),
      D     => WR_Data(8),
      DPRA0 => RD_Addr(0),
      DPRA1 => RD_Addr(1),
      DPRA2 => RD_Addr(2),
      DPRA3 => RD_Addr(3),
      WCLK  => WR_CLK,
      WE    => Stack_WE);

  U9 : RAM16X1D
    generic map (
      INIT => X"0000")
    port map (
      DPO   => RD_Data(9),
      SPO   => RAM_WR_out(9),
      A0    => WR_Addr(0),
      A1    => WR_Addr(1),
      A2    => WR_Addr(2),
      A3    => WR_Addr(3),
      D     => WR_Data(9),
      DPRA0 => RD_Addr(0),
      DPRA1 => RD_Addr(1),
      DPRA2 => RD_Addr(2),
      DPRA3 => RD_Addr(3),
      WCLK  => WR_CLK,
      WE    => Stack_WE);

  U10 : RAM16X1D
    generic map (
      INIT => X"0000")
    port map (
      DPO   => RD_Data(10),
      SPO   => RAM_WR_out(10),
      A0    => WR_Addr(0),
      A1    => WR_Addr(1),
      A2    => WR_Addr(2),
      A3    => WR_Addr(3),
      D     => WR_Data(10),
      DPRA0 => RD_Addr(0),
      DPRA1 => RD_Addr(1),
      DPRA2 => RD_Addr(2),
      DPRA3 => RD_Addr(3),
      WCLK  => WR_CLK,
      WE    => Stack_WE);

  U11 : RAM16X1D
    generic map (
      INIT => X"0000")
    port map (
      DPO   => RD_Data(11),
      SPO   => RAM_WR_out(11),
      A0    => WR_Addr(0),
      A1    => WR_Addr(1),
      A2    => WR_Addr(2),
      A3    => WR_Addr(3),
      D     => WR_Data(11),
      DPRA0 => RD_Addr(0),
      DPRA1 => RD_Addr(1),
      DPRA2 => RD_Addr(2),
      DPRA3 => RD_Addr(3),
      WCLK  => WR_CLK,
      WE    => Stack_WE);

end arch_stack_ram;
