library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library unisim;
use unisim.vcomponents.all;

use work.User_App_comp.all;

entity User_App is
  port (
    CLK_50MHZ_in     : in  std_logic;
    CLK_AUX_in       : in  std_logic;
    Reset_in         : in  std_logic;
    Slider_Switch_in : in  std_logic_vector(7 downto 0);
    PB_Switch_in     : in  std_logic_vector(3 downto 0);
    LED_out          : out std_logic_vector(7 downto 0);
--------------------------------------------------------
--    Mem_Address_out  : out std_logic_vector(15 downto 0);
--    SMP_MDI_out      : out std_logic_vector(15 downto 0);
--    SMP_MDO_out      : out std_logic_vector(15 downto 0);
--    IO_Address_out   : out std_logic_vector(15 downto 0);
--    SMP_IODI_out     : out std_logic_vector(15 downto 0);
--    SMP_IODO_out     : out std_logic_vector(15 downto 0);
--------------------------------------------------------    
    Segment_A_out    : out std_logic;
    Segment_B_out    : out std_logic;
    Segment_C_out    : out std_logic;
    Segment_D_out    : out std_logic;
    Segment_E_out    : out std_logic;
    Segment_F_out    : out std_logic;
    Segment_G_out    : out std_logic;
    Segment_DP_out   : out std_logic;
    Segment_AN_out   : out std_logic_vector(3 downto 0));
end User_App;

architecture arch_User_App of User_App is

  signal CLK_50MHZ     : std_logic;
  signal CLK_AUX       : std_logic;
  signal Reset         : std_logic;
  signal Slider_Switch : std_logic_vector(15 downto 0);
  signal PB_Switch     : std_logic_vector(3 downto 0);
  signal LED           : std_logic_vector(7 downto 0);
  signal Segment_A     : std_logic;
  signal Segment_B     : std_logic;
  signal Segment_C     : std_logic;
  signal Segment_D     : std_logic;
  signal Segment_E     : std_logic;
  signal Segment_F     : std_logic;
  signal Segment_G     : std_logic;
  signal Segment_DP    : std_logic;
  signal Segment_AN    : std_logic_vector(3 downto 0);

  signal CLKDV : std_logic;
  signal Clock : std_logic;

  signal Int         : std_logic;
  signal SMP_MDI     : std_logic_vector(15 downto 0);
  signal SMP_IODI    : std_logic_vector(15 downto 0);
  signal MWait       : std_logic;
  signal IOWait      : std_logic;
  signal Mem_Address : std_logic_vector(15 downto 0);
  signal SMP_MDO     : std_logic_vector(15 downto 0);
  signal MRD         : std_logic;
  signal MWR         : std_logic;
  signal IO_Address  : std_logic_vector(15 downto 0);
  signal SMP_IODO    : std_logic_vector(15 downto 0);
  signal IORD        : std_logic;
  signal IOWR        : std_logic;

  signal Mem_Sel : std_logic;
  signal RAM_WR  : std_logic;
  signal RAM_MDO : std_logic_vector(15 downto 0);
  signal ROM_MDO : std_logic_vector(15 downto 0);

  signal IO_Sel     : std_logic;
  signal DISP_WR    : std_logic;
  signal Int_Gen_RD : std_logic;
  signal DISP_Reg   : std_logic_vector(15 downto 0);
  signal Switch_Reg : std_logic_vector(15 downto 0);
  signal Int_En     : std_logic;
  signal Display_En : std_logic;

begin  -- arch_User_App

  CLK_50MHZ <= CLK_50MHZ_in;
  CLK_AUX   <= CLK_AUX_in;
  Reset     <= Reset_in;

  Slider_Switch(7 downto 0)  <= Slider_Switch_in(7 downto 0);
--  Slider_Switch(15 downto 8) <= Slider_Switch_in(7 downto 0);
  Slider_Switch(15 downto 8) <= X"00";
  
  PB_Switch <= PB_Switch_in;

  LED_out        <= LED;
  Segment_A_out  <= Segment_A;
  Segment_B_out  <= Segment_B;
  Segment_C_out  <= Segment_C;
  Segment_D_out  <= Segment_D;
  Segment_E_out  <= Segment_E;
  Segment_F_out  <= Segment_F;
  Segment_G_out  <= Segment_G;
  Segment_DP_out <= Segment_DP;
  Segment_AN_out <= Segment_AN;

---------------------------------------------------------
--    Mem_Address_out  <= Mem_Address;
--    SMP_MDI_out      <= SMP_MDI;
--    SMP_MDO_out      <= SMP_MDO;
--   IO_Address_out   <= IO_Address;
--    SMP_IODI_out     <= SMP_IODI;
--    SMP_IODO_out     <= SMP_IODO;
---------------------------------------------------------

  CLK_Am2910_BUFG : BUFG
    port map (
      O => Clock,
      I => CLK_50MHZ);
--      I => CLKDV);

--  CLK_50MHZ_DCM : DCM
--    generic map (
--      CLKDV_DIVIDE          => 2.0,
--      CLKFX_DIVIDE          => 1,
--      CLKFX_MULTIPLY        => 4,
--      CLKIN_DIVIDE_BY_2     => true,
--      CLKIN_PERIOD          => 0.0,
--      CLKOUT_PHASE_SHIFT    => "NONE",
--      CLK_FEEDBACK          => "1X",
--      DESKEW_ADJUST         => "SYSTEM_SYNCHRONOUS",
--      DFS_FREQUENCY_MODE    => "LOW",
--      DLL_FREQUENCY_MODE    => "LOW",
--      DUTY_CYCLE_CORRECTION => true,
--      FACTORY_JF            => X"C080",
--      PHASE_SHIFT           => 0,
--      STARTUP_WAIT          => false)
--    port map (
--      CLK0                  => CLKDV,
--      CLK180                => open,
--      CLK270                => open,
--      CLK2x                 => open,
--      CLK2X180              => open,
--      CLK90                 => open,
--      CLKDV                 => open,
--      CLKFX                 => open,
--      CLKFX180              => open,
--      LOCKED                => open,
--      PSDONE                => open,
--      STATUS                => open,
--      CLKFB                 => Clock,
--      CLKIN                 => CLK_50MHZ,
--      PSCLK                 => open,
--      PSEN                  => open,
--      PSINCDEC              => open,
--      RST                   => Reset);

  U1 : smp
    port map (
      CP_in      => Clock,
      Reset_in   => Reset,
      Int_in     => Int,
      MDI_in     => SMP_MDI,
      IODI_in    => SMP_IODI,
      MWait_in   => MWait,
      IOWait_in  => IOWait,
      MAddr_out  => Mem_Address,
      MDO_out    => SMP_MDO,
      MRD_out    => MRD,
      MWR_out    => MWR,
      IOAddr_out => IO_Address,
      IODO_out   => SMP_IODO,
      IORD_out   => IORD,
      IOWR_out   => IOWR);

  U2 : Memory_Decoder
    port map (
      Mem_Address_in => Mem_Address,
      MRD_in         => MRD,
      MWR_in         => MWR,
      Mem_Sel_out    => Mem_Sel,
      MWait_out      => MWait,
      RAM_WR_out     => RAM_WR);

  U3 : RAM_128dx16w
    port map (
      Clock_in   => Clock,
      Address_in => Mem_Address(6 downto 0),
      DI_in      => SMP_MDO,
      WE_in      => RAM_WR,
      DO_out     => RAM_MDO);

  U4 : ROM_128dx16w
    port map (
      Address_in => Mem_Address(6 downto 0),
      DO_out     => ROM_MDO);

  U5 : mux_2to1
    generic map (
      WIDTH         => 16)
    port map (
      D0_in         => ROM_MDO,
      D1_in         => RAM_MDO,
      Mux_Select_in => Mem_Sel,
      Mux_out       => SMP_MDI);

  U6 : IO_Decoder
    port map (
      IO_Address_in  => IO_Address,
      IORD_in        => IORD,
      IOWR_in        => IOWR,
      IO_Sel_out     => IO_Sel,
      IOWait_out     => IOWait,
      DISP_WR_out    => DISP_WR,
      Int_Gen_RD_out => Int_Gen_RD);

  U7 : reg_vbe
    generic map (
      WIDTH     => 16)
    port map (
      Clock_in  => Clock,
      Reset_in  => Reset,
      Enable_in => DISP_WR,
      D_in      => SMP_IODO,
      Q_out     => DISP_Reg);

  U8 : Display_Cntrl
    port map (
      Clock_in       => Clock,
      Reset_in       => Reset,
      Enable_in      => Display_En,
      DI_in          => DISP_Reg,
      Segment_A_out  => Segment_A,
      Segment_B_out  => Segment_B,
      Segment_C_out  => Segment_C,
      Segment_D_out  => Segment_D,
      Segment_E_out  => Segment_E,
      Segment_F_out  => Segment_F,
      Segment_G_out  => Segment_G,
      Segment_DP_out => Segment_DP,
      Segment_AN_out => Segment_AN);

  U9 : Interrupt_Gen
    port map (
      Clock_in      => Clock,
      Reset_in      => Reset,
      Enable_in     => Int_En,
      Int_Gen_RD_in => Int_Gen_RD,
      Int_out       => Int);

  U10 : reg_vbe
    generic map (
      WIDTH     => 16)
    port map (
      Clock_in  => Clock,
      Reset_in  => Reset,
      Enable_in => '1',
      D_in      => Slider_Switch,
      Q_out     => Switch_Reg);

  U11 : mux_2to1
    port map (
      D0_in         => DISP_Reg,
      D1_in         => Switch_Reg,
      Mux_Select_in => IO_Sel,
      Mux_out       => SMP_IODI);

  U12 : Clock_Gen
    port map (
      Clock_in       => Clock,
      Reset_in       => Reset,
      Int_En_out     => Int_En,
      Display_En_out => Display_En);

  U13 : process (Clock, Reset)
  begin  -- process U13
    if Reset = '1' then

      LED <= "00000000";

    elsif Clock'event and Clock = '1' then

	if Int_En = '1' then 
		LED(0) <= not LED(0);
	end if;

	if Display_En = '1' then
		LED(1) <= not LED(1);
	end if;

	if MRD = '1' then
		LED(2) <= not LED(2);
	end if;

	if MWR = '1' then
		LED(3) <= not LED(3);
	end if;

	if IORD = '1' then
		LED(4) <= not LED(4);
	end if;

	if IOWR = '1' then
		LED(5) <= not LED(5);
	end if;
	
	if INT_GEN_RD = '1' then
		LED(6) <= not LED(6);
	end if;
	
	if DISP_WR = '1' then
		LED(7) <= not LED(7);
	end if;
	
--      if Int_En = '1' then
--	if MRD = '1' then
        
--        	LED(6 downto 0) <= Mem_Address(6 downto 0);
--        	LED(7) <= Mem_Address(15);
	
--	end if; -- if MRD = '1'
--      end if;  -- if Display_En = '1'
    end if;  -- if Reset = '1'

  end process U13;

end arch_User_App;
