library ieee;

use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

package User_App_comp is

  component smp
    port (
      CP_in      : in  std_logic;
      Reset_in   : in  std_logic;
      Int_in     : in  std_logic;
      MDI_in     : in  std_logic_vector(15 downto 0);
      IODI_in    : in  std_logic_vector(15 downto 0);
      MWait_in   : in  std_logic;
      IOWait_in  : in  std_logic;
      MAddr_out  : out std_logic_vector(15 downto 0);
      MDO_out    : out std_logic_vector(15 downto 0);
      MRD_out    : out std_logic;
      MWR_out    : out std_logic;
      IOAddr_out : out std_logic_vector(15 downto 0);
      IODO_out   : out std_logic_vector(15 downto 0);
      IORD_out   : out std_logic;
      IOWR_out   : out std_logic);
  end component;

  component RAM_128dx16w
    port (
      Clock_in   : in  std_logic;
      Address_in : in  std_logic_vector(6 downto 0);
      DI_in      : in  std_logic_vector(15 downto 0);
      WE_in      : in  std_logic;
      DO_out     : out std_logic_vector(15 downto 0));
  end component;

  component ROM_128dx16w
    port (
      Address_in : in  std_logic_vector(6 downto 0);
      DO_out     : out std_logic_vector(15 downto 0));
  end component;

  component Memory_Decoder
    port (
      Mem_Address_in : in  std_logic_vector(15 downto 0);
      MRD_in         : in  std_logic;
      MWR_in         : in  std_logic;
      Mem_Sel_out    : out std_logic;
      MWait_out      : out std_logic;
      RAM_WR_out     : out std_logic);
  end component;

  component IO_Decoder
    port (
      IO_Address_in  : in  std_logic_vector(15 downto 0);
      IORD_in        : in  std_logic;
      IOWR_in        : in  std_logic;
      IO_Sel_out     : out std_logic;
      IOWait_out     : out std_logic;
      DISP_WR_out    : out std_logic;
      Int_Gen_RD_out : out std_logic);
  end component;

  component mux_2to1
    generic (
      WIDTH         :     integer := 16);
    port (
      D0_in         : in  std_logic_vector((WIDTH - 1) downto 0);
      D1_in         : in  std_logic_vector((WIDTH - 1) downto 0);
      Mux_Select_in : in  std_logic;
      Mux_out       : out std_logic_vector((WIDTH - 1) downto 0));
  end component;

  component reg_vbe
    generic (
      WIDTH     :     integer);
    port (
      Clock_in  : in  std_logic;
      Reset_in  : in  std_logic;
      Enable_in : in  std_logic;
      D_in      : in  std_logic_vector((WIDTH - 1) downto 0);
      Q_out     : out std_logic_vector((WIDTH - 1) downto 0));
  end component;

  component Interrupt_Gen
    port (
      Clock_in      : in  std_logic;
      Reset_in      : in  std_logic;
      Enable_in     : in  std_logic;
      Int_Gen_RD_in : in  std_logic;
      Int_out       : out std_logic);
  end component;

  component Display_Cntrl
    port (
      Clock_in       : in  std_logic;
      Reset_in       : in  std_logic;
      Enable_in      : in  std_logic;
      DI_in          : in  std_logic_vector(15 downto 0);
      Segment_A_out  : out std_logic;
      Segment_B_out  : out std_logic;
      Segment_C_out  : out std_logic;
      Segment_D_out  : out std_logic;
      Segment_E_out  : out std_logic;
      Segment_F_out  : out std_logic;
      Segment_G_out  : out std_logic;
      Segment_DP_out : out std_logic;
      Segment_AN_out : out std_logic_vector(3 downto 0));
  end component;

  component Clock_Gen
    port (
      Clock_in       : in  std_logic;
      Reset_in       : in  std_logic;
      Int_En_out     : out std_logic;
      Display_En_out : out std_logic);
  end component;

end User_App_comp;
