library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

library unisim;
use unisim.vcomponents.all;

use work.S3_IF_comp.all;

entity S3_IF is

  port (
    CLK_50MHZ_pin     : in  std_logic;
    CLK_AUX_pin       : in  std_logic;
    Slider_Switch_pin : in  std_logic_vector(7 downto 0);
    PB_Switch_pin     : in  std_logic_vector(3 downto 0);
    LED_pin           : out std_logic_vector(7 downto 0);
    Segment_A_pin     : out std_logic;
    Segment_B_pin     : out std_logic;
    Segment_C_pin     : out std_logic;
    Segment_D_pin     : out std_logic;
    Segment_E_pin     : out std_logic;
    Segment_F_pin     : out std_logic;
    Segment_G_pin     : out std_logic;
    Segment_DP_pin    : out std_logic;
    Segment_AN_pin    : out std_logic_vector(3 downto 0));
end S3_IF;

architecture arch_S3_IF of S3_IF is

  signal CLK_50MHZ     : std_logic;
  signal CLK_AUX       : std_logic;
  signal Slider_Switch : std_logic_vector(7 downto 0);
  signal PB_Switch     : std_logic_vector(3 downto 0);
  signal LED           : std_logic_vector(7 downto 0);
  signal Segment_A     : std_logic;
  signal Segment_B     : std_logic;
  signal Segment_C     : std_logic;
  signal Segment_D     : std_logic;
  signal Segment_E     : std_logic;
  signal Segment_F     : std_logic;
  signal Segment_G     : std_logic;
  signal Segment_DP    : std_logic;
  signal Segment_AN    : std_logic_vector(3 downto 0);

  signal Reset : std_logic;

begin  -- arch_S3_IF

  SPARTAN_ROC : ROC
    generic map (
      WIDTH => 400 ns)
    port map (
      O     => Reset);

  U1 : CLK_IF
    port map (
      CLK_50MHZ_input  => CLK_50MHZ_pin,
      CLK_AUX_input    => CLK_AUX_pin,
      CLK_50MHZ_output => CLK_50MHZ,
      CLK_AUX_output   => CLK_AUX);

  U3 : SW_IF
    port map (
      Slider_Switch_input  => Slider_Switch_pin,
      PB_Switch_input      => PB_Switch_pin,
      Slider_Switch_output => Slider_Switch,
      PB_Switch_output     => PB_Switch);

  U4 : LED_IF
    port map (
      LED_input  => LED,
      LED_output => LED_pin);

  U5 : DISP_IF
    port map (
      Segment_A_input   => Segment_A,
      Segment_B_input   => Segment_B,
      Segment_C_input   => Segment_C,
      Segment_D_input   => Segment_D,
      Segment_E_input   => Segment_E,
      Segment_F_input   => Segment_F,
      Segment_G_input   => Segment_G,
      Segment_DP_input  => Segment_DP,
      Segment_AN_input  => Segment_AN,
      Segment_A_output  => Segment_A_pin,
      Segment_B_output  => Segment_B_pin,
      Segment_C_output  => Segment_C_pin,
      Segment_D_output  => Segment_D_pin,
      Segment_E_output  => Segment_E_pin,
      Segment_F_output  => Segment_F_pin,
      Segment_G_output  => Segment_G_pin,
      Segment_DP_output => Segment_DP_pin,
      Segment_AN_output => Segment_AN_pin);

  U6 : User_App
    port map (
      CLK_50MHZ_in     => CLK_50MHZ,
      CLK_AUX_in       => CLK_AUX,
      Reset_in         => Reset,
      Slider_Switch_in => Slider_Switch,
      PB_Switch_in     => PB_Switch,
      LED_out          => LED,
      Segment_A_out    => Segment_A,
      Segment_B_out    => Segment_B,
      Segment_C_out    => Segment_C,
      Segment_D_out    => Segment_D,
      Segment_E_out    => Segment_E,
      Segment_F_out    => Segment_F,
      Segment_G_out    => Segment_G,
      Segment_DP_out   => Segment_DP,
      Segment_AN_out   => Segment_AN);

end arch_S3_IF;
