library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.smp_const.all;

entity shift_cntrl is

  port (
    Q_LSB_in       : in  std_logic;
    Q_MSB_in       : in  std_logic;
    RAM_LSB_in     : in  std_logic;
    RAM_MSB_in     : in  std_logic;
    Carry_in       : in  std_logic;
    I_in           : in  std_logic_vector(2 downto 0);
    Q_LSB_out      : out std_logic;
    Q_MSB_out      : out std_logic;
    RAM_LSB_out    : out std_logic;
    RAM_MSB_out    : out std_logic;
    Carry_out      : out std_logic;
    Carry_Load_out : out std_logic);
end shift_cntrl;

architecture arch_shift_cntrl of shift_cntrl is

  signal I          : std_logic_vector(2 downto 0);
  signal Carry_Load : std_logic;

begin  -- arch_shift_cntrl

  I <= I_in;

  Carry_Load_out <= Carry_Load;

  Q_MSB_out <= Carry_in when I = RSHFTA else
               Q_LSB_in when I = RROT   else
               '0';

  Q_LSB_out <= Q_MSB_in when I = LROT else
               '0';

  RAM_MSB_out <= Carry_in   when I = RSHFTA else
                 RAM_LSB_in when I = RROT   else
                 '0';

  RAM_LSB_out <= RAM_MSB_in when I = LROT else
                 '0';

  Carry_out <= Carry_in when I = LSHFT  else
               Carry_in when I = LSHFTA else
               '0';

  Carry_Load <= '1' when I = LSHFT  else
                '1' when I = LSHFTA else
                '0';

end arch_shift_cntrl;
