library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity reg_vbe is

  generic (
    WIDTH     :     integer := 16);
  port (
    Clock_in  : in  std_logic;
    Reset_in  : in  std_logic;
    Enable_in : in  std_logic;
    D_in      : in  std_logic_vector((WIDTH - 1) downto 0);
    Q_out     : out std_logic_vector((WIDTH - 1) downto 0));

end reg_vbe;

architecture arch_reg_vbe of reg_vbe is

  signal Clock  : std_logic;
  signal Reset  : std_logic;
  signal Enable : std_logic;
  signal D      : std_logic_vector((WIDTH - 1) downto 0);
  signal Q      : std_logic_vector((WIDTH - 1) downto 0);

begin  -- arch_reg_vbe

  Clock  <= Clock_in;
  Reset  <= Reset_in;
  Enable <= Enable_in;
  D      <= D_in;

  Q_out <= Q;

  U1 : process (Clock, Reset)
  begin  -- process Clock
    if Reset = '1' then                 -- asynchronous reset (active high)

      Q <= (others => '0');

    elsif Clock'event and Clock = '1' then  -- rising clock edge
      if Enable = '1' then

        Q <= D;

      end if;  -- if Enable = '1'
    end if;  -- if Reset = '1'

  end process U1;

end arch_reg_vbe;
