library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use ieee.std_logic_arith.all;

package smp_comp is

  component Am2901
    generic (
      SLICE_WIDTH     :     integer := 4);
    port (
      CP_in           : in  std_logic;
      A_in            : in  std_logic_vector(3 downto 0);
      B_in            : in  std_logic_vector(3 downto 0);
      D_in            : in  std_logic_vector((SLICE_WIDTH - 1) downto 0);
      I_in            : in  std_logic_vector(8 downto 0);
      nOE_in          : in  std_logic;
      CI_in           : in  std_logic;
      Q_LSB_in        : in  std_logic;
      Q_MSB_in        : in  std_logic;
      RAM_LSB_in      : in  std_logic;
      RAM_MSB_in      : in  std_logic;
      Y_out           : out std_logic_vector((SLICE_WIDTH - 1) downto 0);
      nG_out          : out std_logic;
      nP_out          : out std_logic;
      Feq0_out        : out std_logic;
      F_MSB_out       : out std_logic;
      OVR_out         : out std_logic;
      CO_out          : out std_logic;
      Q_LSB_out       : out std_logic;
      Q_MSB_out       : out std_logic;
      Q_LSB_Dir_out   : out std_logic;
      Q_MSB_Dir_out   : out std_logic;
      RAM_LSB_out     : out std_logic;
      RAM_MSB_out     : out std_logic;
      RAM_LSB_Dir_out : out std_logic;
      RAM_MSB_Dir_out : out std_logic);
  end component;

  component am2910
    generic (
      STACK_DEPTH  :     integer);
    port (
      CP_in        : in  std_logic;     -- clock input
      nCC_in       : in  std_logic;     -- condition code
      nCCEN_in     : in  std_logic;     -- condition code enable
      CI_in        : in  std_logic;     -- carry in (to incrementer)
      D_in         : in  std_logic_vector(11 downto 0);  -- direct input
      I_in         : in  std_logic_vector(3 downto 0);  -- instruction
      nOE_in       : in  std_logic;     -- output enable for y output
      nRLD_in      : in  std_logic;     -- register/counter load enable
      nMap_En_out  : out std_logic;     -- mapping prom output enable
      nPL_En_out   : out std_logic;     -- pipeline register output enable
      nVect_En_out : out std_logic;     -- vector output enable
      Y_out        : out std_logic_vector(11 downto 0);  -- microprogram address
      nFull_out    : out std_logic);    -- stack full flag
  end component;

  component Control_Store
    generic (
      WIDTH    :     integer;
      DEPTH    :     integer);
    port (
      CP_in    : in  std_logic;
      Reset_in : in  std_logic;
      WE_in    : in  std_logic;
      EN_in    : in  std_logic;
      Addr_in  : in  std_logic_vector(11 downto 0);
      DI_in    : in  std_logic_vector((WIDTH - 1) downto 0);
      DO_out   : out std_logic_vector((WIDTH - 1) downto 0));
  end component;

  component mux_2to1b
    port (
      D0_in         : in  std_logic;
      D1_in         : in  std_logic;
      Mux_Select_in : in  std_logic;
      Mux_out       : out std_logic);
  end component;

  component mux_2to1
    generic (
      WIDTH         :     integer := 16);
    port (
      D0_in         : in  std_logic_vector((WIDTH - 1) downto 0);
      D1_in         : in  std_logic_vector((WIDTH - 1) downto 0);
      Mux_Select_in : in  std_logic;
      Mux_out       : out std_logic_vector((WIDTH - 1) downto 0));
  end component;

  component mux_4to1
    generic (
      WIDTH         :     integer := 16);
    port (
      D0_in         : in  std_logic_vector((WIDTH - 1) downto 0);
      D1_in         : in  std_logic_vector((WIDTH - 1) downto 0);
      D2_in         : in  std_logic_vector((WIDTH - 1) downto 0);
      D3_in         : in  std_logic_vector((WIDTH - 1) downto 0);
      Mux_Select_in : in  std_logic_vector(1 downto 0);
      Mux_out       : out std_logic_vector((WIDTH - 1) downto 0));
  end component;

  component mux_8to1b
    port (
      D0_in         : in  std_logic;
      D1_in         : in  std_logic;
      D2_in         : in  std_logic;
      D3_in         : in  std_logic;
      D4_in         : in  std_logic;
      D5_in         : in  std_logic;
      D6_in         : in  std_logic;
      D7_in         : in  std_logic;
      Mux_Select_in : in  std_logic_vector(2 downto 0);
      Mux_out       : out std_logic);
  end component;

  component Shift_Cntrl
    port (
      Q_LSB_in       : in  std_logic;
      Q_MSB_in       : in  std_logic;
      RAM_LSB_in     : in  std_logic;
      RAM_MSB_in     : in  std_logic;
      Carry_in       : in  std_logic;
      I_in           : in  std_logic_vector(2 downto 0);
      Q_LSB_out      : out std_logic;
      Q_MSB_out      : out std_logic;
      RAM_LSB_out    : out std_logic;
      RAM_MSB_out    : out std_logic;
      Carry_out      : out std_logic;
      CARRY_Load_out : out std_logic);
  end component;

  component Stat_Reg
    port (
      Clock_in       : in  std_logic;
      Reset_in       : in  std_logic;
      Zero_in        : in  std_logic;
      Carry_in       : in  std_logic;
      Sign_in        : in  std_logic;
      Overflow_in    : in  std_logic;
      D_in           : in  std_logic_vector(3 downto 0);
      Shift_Carry_in : in  std_logic;
      SR_Load_in     : in  std_logic;
      SRC_Load_in    : in  std_logic;
      SR_out         : out std_logic_vector(3 downto 0));
  end component;

  component IR_Map_Mux
    port (
      Clock_in             : in  std_logic;
      Reset_in             : in  std_logic;
      IR_in                : in  std_logic_vector(15 downto 0);
      IR_Map_Mux_Select_in : in  std_logic_vector(1 downto 0);
      IR_Map_out           : out std_logic_vector(11 downto 0));
  end component;

  component reg_vbe
    generic (
      WIDTH     :     integer);
    port (
      Clock_in  : in  std_logic;
      Reset_in  : in  std_logic;
      Enable_in : in  std_logic;
      D_in      : in  std_logic_vector((WIDTH - 1) downto 0);
      Q_out     : out std_logic_vector((WIDTH - 1) downto 0));
  end component;

  component reg_1be
    port (
      Clock_in  : in  std_logic;
      Reset_in  : in  std_logic;
      Enable_in : in  std_logic;
      D_in      : in  std_logic;
      Q_out     : out std_logic);
  end component;

end package;
